if (not nUI) then return end

local ncc = {
	silent = false,
}

nUI_Config_Colors = ncc

nUI_Config_Colors_Options = ncc.db
if (not ncc.frame) then
	ncc.frame = CreateFrame("Frame")
end

local function split(str, pat, trim)
    local ftrim = function (s) return s end
    if (not trim) then trim = true end
    -- if (trim == true) then ftrim = function (s) return self:Trim(s) end end

    local t = {}  -- NOTE: use {n = 0} in Lua-5.0
    local fpat = "(.-)" .. pat
    local last_end = 1
    local s, e, cap = string.find(str, fpat, 1)
    while s do
        if s ~= 1 or cap ~= "" then
            table.insert(t, ftrim(cap))
        end
        last_end = e+1
        s, e, cap = string.find(str, fpat, last_end)
    end
    if last_end <= string.len(str) then
        cap = string.sub(str, last_end)
        table.insert(t, ftrim(cap))
    end
    
    return t
end

local function explode(str, pat, trim)
 	return unpack(split(str, pat or ",", trim))
end

local function setColors(name, colors)
	if (name == nil) then return end
	if (colors == nil) then return end
	if (colors.r == nil) or (colors.g == nil) or (colors.b == nil) then return end
	
	if (not nUI_ButtonColors[name]) then return end
	
	nUI_ButtonColors[name] = colors
	ncc.db.buttonColors[name] = nUI_ButtonColors[name]
	return true
end

local function setColorsOOR(colors)
	return setColors("OOR", colors)
end

local function setColorsOOM(colors)
	return setColors("OOM", colors)
end

local function setColorsUNUSABLE(colors)
	return setColors("UNUSABLE", colors)
end

local function setColorsCDC1(colors)
	return setColors("CDC1", colors)
end

local function setColorsCDC2(colors)
	return setColors("CDC2", colors)
end

local function setColorsRESET()
	nUI_ButtonColors["OOR"] = nUI_DefaultConfig.ButtonColors["OOR"]
	nUI_ButtonColors["OOM"] = nUI_DefaultConfig.ButtonColors["OOM"]
	nUI_ButtonColors["UNUSABLE"] = nUI_DefaultConfig.ButtonColors["UNUSABLE"]	
	nUI_ButtonColors["CDC1"] = nUI_DefaultConfig.ButtonColors["CDC1"]
	nUI_ButtonColors["CDC2"] = nUI_DefaultConfig.ButtonColors["CDC2"]
	
	ncc.db.buttonColors = nUI_ButtonColors
end

local function initSlashCommands()
	-- nUI_L["nUI: The value [ %s ] is not a valid font size. Please choose a number between 5 and 30"] = "nUI: The value [ %s ] is not a valid font size. Please choose a number between 5 and 30"

	nUI_SLASHCMD_COLORS = #nUI_SlashCommands + 1
	nUI_SLASHCMD_COLORS_RESET = 1
	nUI_SLASHCMD_COLORS_OOR = 2
	nUI_SLASHCMD_COLORS_OOM = 3
	nUI_SLASHCMD_COLORS_UNUSABLE = 4
	nUI_SLASHCMD_COLORS_CDC1 = 5
	nUI_SLASHCMD_COLORS_CDC2 = 6
	
	nUI_SlashCommands[nUI_SLASHCMD_COLORS] =
	{
		command = "colors",
		options = nil,
		desc    = "Changes color options",
		message = nil,
		sub_menu = {
			[nUI_SLASHCMD_COLORS_RESET] = {
				command = "reset",
				options = nil,
				desc = "Reset color values to default",
				message = nil,
			},

			[nUI_SLASHCMD_COLORS_OOR] = {
				command = "oor",
				options = "{n,n,n}",
				desc = "Colors for Out Of Reach",
				message = nil,
			},

			[nUI_SLASHCMD_COLORS_OOM] = {
				command = "oom",
				options = "{n,n,n}",
				desc = "Colors for Out Of Mana",
				message = nil,
			},

			[nUI_SLASHCMD_COLORS_UNUSABLE] = {
				command = "unusable",
				options = "{n,n,n}",
				desc = "Colors for Unusable",
				message = nil,
			},
			
		},
	}
	
	local master = nUI_SlashCommands[nUI_SLASHCMD_COLORS]
	local option = master.sub_menu[nUI_SLASHCMD_COLORS_RESET]
	nUI_SlashCommands:setSubHandler(master.command, option.command,
		function(msg)
			setColorsRESET()
		end
	)

	local option = master.sub_menu[nUI_SLASHCMD_COLORS_OOR]
	nUI_SlashCommands:setSubHandler( master.command, option.command,
		function(msg, arg1)
			if (arg1 ~= nil) then
				local red, green, blue = explode(arg1)
				local colors = {
					r = red,
					g = green,
					b = blue,
				}
				
				setColorsOOR(colors)
			end
			
			local c = nUI_ButtonColors["OOR"]
			local r, g, b = c.r, c.g, c.b
			DEFAULT_CHAT_FRAME:AddMessage(("Out Of Reach colors are Red: %s; Green: %s; Blue: %s"):format(r, g, b))
			
		end
	)
		
	local option = master.sub_menu[nUI_SLASHCMD_COLORS_OOM]
	nUI_SlashCommands:setSubHandler( master.command, option.command,
		function(msg, arg1)
			if (arg1 ~= nil) then
				local red, green, blue = explode(arg1)
				local colors = {
					r = red,
					g = green,
					b = blue,
				}
				
				setColorsOOM(colors)
			end

			local c = nUI_ButtonColors["OOM"]
			local r, g, b = c.r, c.g, c.b
			DEFAULT_CHAT_FRAME:AddMessage(("Out Of Mana colors are Red: %s; Green: %s; Blue: %s"):format(r, g, b))
			
		end
	)
	
	local option = master.sub_menu[nUI_SLASHCMD_COLORS_UNUSABLE]
	nUI_SlashCommands:setSubHandler( master.command, option.command,
		function(msg, arg1)
			if (arg1 ~= nil) then
				local red, green, blue = explode(arg1)
				local colors = {
					r = red,
					g = green,
					b = blue,
				}
				
				setColorsUNUSABLE(colors)
			end
			
			local c = nUI_ButtonColors["UNUSABLE"]
			local r, g, b = c.r, c.g, c.b
			DEFAULT_CHAT_FRAME:AddMessage(("Unusable colors are Red: %s; Green: %s; Blue: %s"):format(r, g, b))
			
		end
	)


end

local function initConfig()
	if (not nUIConfigGUI) then return end
	ncc.silent = true
	
	local function getConfigColors()
		local colors = {
			type = "group",
			name = "Color options",
			desc = "Color options",
			args = {
				__oor = {
					type = "color",
					name = "Out Of Reach",
					desc = "Out Of Reach",
					get = function()
						local r = nUI_ButtonColors["OOR"].r
						local g = nUI_ButtonColors["OOR"].g
						local b = nUI_ButtonColors["OOR"].b

						return r, g, b
					end,
					set = function(t,a,b,c,d)
						local colors = {
							r = a,
							g = b,
							b = c,
						}
						
						setColorsOOR(colors)
					end,
					order = 1,
				},
				
				__oom = {
					type = "color",
					name = "Out Of Mana",
					desc = "Out Of Mana",
					get = function()
						local r = nUI_ButtonColors["OOM"].r
						local g = nUI_ButtonColors["OOM"].g
						local b = nUI_ButtonColors["OOM"].b
						
						return r, g, b
					end,
					set = function(t,a,b,c,d)
						local colors = {
							r = a,
							g = b,
							b = c,
						}
						
						setColorsOOM(colors)
					end,
					order = 2,
				},

				__unusable = {
					type = "color",
					name = "Unusable",
					desc = "Unusable",
					get = function()
						local r = nUI_ButtonColors["UNUSABLE"].r
						local g = nUI_ButtonColors["UNUSABLE"].g
						local b = nUI_ButtonColors["UNUSABLE"].b
						
						return r, g, b
					end,
					set = function(t,a,b,c,d)
						local colors = {
							r = a,
							g = b,
							b = c,
						}
						
						setColorsUNUSABLE(colors)
					end,
					order = 3,
				},
				
				__reset = {
					type = "execute",
					name = "Reset",
					desc = "Reset to defaults",
					func = setColorsRESET,
					width = "full",
					order = 100,
				}
			},
		}
		
		return colors
	end
	
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")
	
	AceConfigReg:RegisterOptionsTable("nUI Colors",getConfigColors)
	nUIConfigGUI.optionsFrame["Colors"] = AceConfigDialog:AddToBlizOptions("nUI Colors", "Colors", "nUI")		
	
end

local function OnEvent(self, event, arg1)
	if (event == "VARIABLES_LOADED") then
		if (not ncc.silent) then
			DEFAULT_CHAT_FRAME:AddMessage("nUI: Config [Colors] loaded.")
		end

		initSlashCommands()
		initConfig()
		
		ncc.frame:UnregisterEvent("VARIABLES_LOADED")
		
	elseif (event == "ADDON_LOADED") and (arg1 == "nUI_Config_Colors") then
		if (not nUI_DefaultConfig.ButtonColors["UNUSABLE"]) then
			nUI_DefaultConfig.ButtonColors["UNUSABLE"] = nUI_DefaultConfig.ButtonColors["OOR"]
		end
		
		if (not nUI_ButtonColors["UNUSABLE"]) then
			nUI_ButtonColors["UNUSABLE"] = nUI_DefaultConfig.ButtonColors["UNUSABLE"]
		end
		
		ncc.db = nUI_Config_Colors_Options
		if (not ncc.db) then 
			ncc.db = {
				version = "40000.1",
				buttonColors = nUI_ButtonColors,
			}
		end
		
		if (ncc.db.version ~= "40000.1") then
			ncc.db.version = "40000.1"
			ncc.db.buttonColors = nUI_ButtonColors
		end
		
		nUI_Config_Colors_Options = ncc.db
		
		ncc.frame:UnregisterEvent("ADDON_LOADED")
		
	elseif (event == "PLAYER_LOGIN") then
		nUI_ButtonColors = ncc.db.buttonColors
		-- ncc.frame:UnregisterEvent("PLAYER_LOGIN")
		
	end
end

ncc.frame:SetScript("OnEvent", OnEvent)
ncc.frame:RegisterEvent("ADDON_LOADED")
ncc.frame:RegisterEvent("VARIABLES_LOADED")
ncc.frame:RegisterEvent("PLAYER_LOGIN")
